/*  Board: WeMosD1
    sensor : MQ2 & DHT22
    ServiceS: MQTT client, OTA, RemoteDebug

    usato per comunicare con l'accessorio HomeKit AirQualityLab

    controlla i valori ogni 10 sec e li inoltra via MQTT alla richiesta

*/
#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include "RemoteDebug.h"        //https://github.com/JoaoLopesF/RemoteDebug
RemoteDebug Debug;
#include <PubSubClient.h>
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"
#include <dht.h>

#define DHTPIN 2 //D4
#define HOST_NAME "TemperatureSnsr"



dht DHT;
WiFiClient TemperatureSnsr;
PubSubClient client(TemperatureSnsr);

//WiFi settings
const char* ssid = "YourWirelessNameHere";
const char* password = "YourWiFiPassword";
/*MQTT Settings*/
const char* mqtt_server = "192.168.1.123";   // INDIRIZZO IP DEL VOSTRO BROKER MQTT (Raspberry Pi)

const char* inTopic_Humidity =      "HOME/get/Humidity";
const char* inTopic_Temperature =   "HOME/get/Temperature";
const char* outTopic_Temperature =  "HOME/Current/Temperature";
const char* outTopic_Humidity =     "HOME/Current/Humidity";

const char* questMsg = "???";
char msg[50];

unsigned long previousMillis = 0;
uint32_t mLastTime = 0;
uint32_t mTimeSeconds = 0;
float readH, readT, cTemperature, cHumidity;

//********************************************************************************************** CALLBACK
void callback(char* topic, byte* payload, unsigned int length) {
  payload[length] = '\0'; // Null terminator used to terminate the char array
  String message = (char*)payload;
  Debug.print("Message arrived from topic ["); 
  Debug.print(topic);  
  Debug.print("] ["); 
  Debug.print(message);  
  Debug.print("]");  
  Debug.println("");
  
  if (strcmp (topic, inTopic_Humidity) == 0) {
    if (message == questMsg) {     
      Debug.print("***\t Sending Humidity Value:"); Debug.println(cHumidity);
      dtostrf(cHumidity , 2, 2, msg);
      client.publish(outTopic_Humidity, msg);
    }
  }
  if (strcmp (topic, inTopic_Temperature) == 0) {
    if (message == questMsg) {     
      Debug.print("***\t Sending Temperature Value:"); Debug.println(cTemperature);
      dtostrf(cTemperature , 2, 2, msg);
      client.publish(outTopic_Temperature, msg);
    }
  }
  Debug.println(" ");
}
//********************************************************************************************** RECONNET
void reconnect() {
  while (!client.connected()) {
    Debug.println("Attempting MQTT connection...");
    if (client.connect(HOST_NAME)) {
      Debug.println("connected");
      
      client.subscribe(inTopic_Temperature);
      client.subscribe(inTopic_Humidity);
    } else {
      Debug.print("failed, rc=");
      Debug.print(client.state());
      Debug.println(" try again in 5 second");
      delay(5000);
    }
  }
}


void setup() {
  Serial.begin(115200);
  Debug.begin(HOST_NAME);
  Debug.setSerialEnabled(true);
  Debug.setResetCmdEnabled(true);
  ConnectWifi();
  while (WiFi.waitForConnectResult() != WL_CONNECTED) {
    delay (5000);
    ConnectWifi();
  }
  OTA();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
  getTemperature();
  
}

void loop() {
  if (WiFi.waitForConnectResult() != WL_CONNECTED)
    ConnectWifi();
  if (!client.connected())
    reconnect();
  ArduinoOTA.handle();
  Debug.handle();
  if ((millis() - mLastTime) >= 1000) {
    mLastTime = millis();
    mTimeSeconds++;
    // Debug the time (verbose level)
    rdebugVln("* Time: %u seconds (VERBOSE)", mTimeSeconds);

    if (mTimeSeconds % 15 == 0) { // Each 15 seconds
      // Debug levels
      rdebugVln("* This is a message of debug level VERBOSE");
      rdebugDln("* This is a message of debug level DEBUG");
      rdebugIln("* This is a message of debug level INFO");
      rdebugWln("* This is a message of debug level WARNING");
      rdebugEln("* This is a message of debug level ERROR");
    }
    if (mTimeSeconds % 7 == 0) {   // Each 7 seconds
      getTemperature();
    }
  }
  client.loop();
}
bool ConnectWifi() {
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);
  WiFi.begin(ssid, password);
}

void getTemperature() {
  int chk = DHT.read22(DHTPIN);
  readH = DHT.humidity;
  readT = DHT.temperature - 2.0;
  // Check if any reads failed and exit early (to try again).
  if (isnan(readH) || isnan(readT)) {
    Debug.println("* * * * * * * * Failed to read from DHT sensor!* * * * * * * * *");
    Debug.print("R_Temperature "); Debug.print(readT); Debug.print(" R_Humidity "); Debug.println(readH);
    Debug.print("Temperature "); Debug.print(cTemperature); Debug.print(" Humidity "); Debug.println(cHumidity);
    Debug.println("* * * * * * * *                                * * * * * * * * *");
    return;
  } else {
    cTemperature = readT;
    cHumidity = readH;
    Debug.println("++++++++++++++++Positive read from DHT sensor!++++++++++++++++");
    Debug.print("R_Temperature "); Debug.print(readT); Debug.print(" R_Humidity "); Debug.println(readH);
    Debug.print("Temperature "); Debug.print(cTemperature); Debug.print(" Humidity "); Debug.println(cHumidity);
    Debug.println("++++++++++++++++                              ++++++++++++++++");
  }
}
void OTA() {
  ArduinoOTA.setPort(8266);
  ArduinoOTA.setHostname(HOST_NAME);
  //ArduinoOTA.setPassword((const char *)"123");
  ArduinoOTA.onStart([]() {
    Debug.println("Start");
  });
  ArduinoOTA.onEnd([]() {
    Debug.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Debug.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Debug.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) Debug.println("Auth Failed");
    else if (error == OTA_BEGIN_ERROR) Debug.println("Begin Failed");
    else if (error == OTA_CONNECT_ERROR) Debug.println("Connect Failed");
    else if (error == OTA_RECEIVE_ERROR) Debug.println("Receive Failed");
    else if (error == OTA_END_ERROR) Debug.println("End Failed");
  });
  ArduinoOTA.begin();
}
